#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>


static DCB SerialPortStruct;
static HANDLE serial_port;

HANDLE OpenSerial(int idx)
{
	TCHAR comname[100];
	wsprintf(comname, TEXT("\\\\.\\COM%d"), idx);
	HANDLE hComm = CreateFile(comname, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hComm == INVALID_HANDLE_VALUE)
		return NULL;
	COMMTIMEOUTS timeouts = { 0 };
	timeouts.ReadIntervalTimeout = 0x7ffffff;
	timeouts.ReadTotalTimeoutConstant = 0x7ffffff;
	timeouts.ReadTotalTimeoutMultiplier = 0x7ffffff;
	timeouts.WriteTotalTimeoutConstant = 0x7ffffff;
	timeouts.WriteTotalTimeoutMultiplier = 0x7ffffff;

	if (SetCommTimeouts(hComm, &timeouts) == FALSE) return NULL;
	if (SetCommMask(hComm, EV_RXCHAR) == FALSE) return NULL;

	return hComm;
}

static inline int GetStatus()
{
	SerialPortStruct.DCBlength = sizeof(SerialPortStruct);
	return GetCommState(serial_port, &SerialPortStruct);
}

int SendSerialData(const char *data, int len)
{
	DWORD BytesWritten;
	if (WriteFile(serial_port, data, len, &BytesWritten, NULL) == FALSE) return -1;
	return BytesWritten;
}

int ReceiveSerialData(char *data, int len)
{
	DWORD EventMask;

	if (WaitCommEvent(serial_port, &EventMask, NULL))
	{
		DWORD BytesRead;

		return ReadFile(serial_port, data, len, &BytesRead, NULL);
	}
	
	return 0;
}

int SetSerialBoudRate(int rate)
{
	if (GetStatus())
	{
		SerialPortStruct.BaudRate = rate;
		return SetCommState(serial_port, &SerialPortStruct);
	}

	return 0;
}

int GetSerialBoudRate(HANDLE serial_port)
{
	if (GetStatus())
	{
		return SerialPortStruct.BaudRate;
	}

	return -1;
}

int SetSerialStopBits(int bits)
{
	if (GetStatus())
	{
		SerialPortStruct.StopBits = bits;
		return SetCommState(serial_port, &SerialPortStruct);
	}

	return 0;
}

int GetSerialStopBits()
{
	if (GetStatus())
	{
		return SerialPortStruct.StopBits;
	}

	return -1;
}

int GetSerialParity()
{
	if (GetStatus())
	{
		return SerialPortStruct.Parity;
	}

	return -1;
}

int SetSerialParity(int parity)
{
	if (GetStatus())
	{
		SerialPortStruct.Parity = parity;

		return SetCommState(serial_port, &SerialPortStruct);
	}
	
	return 0;
}

int GetSerialDataBits()
{
	if (GetStatus())
	{
		return SerialPortStruct.ByteSize;
	}

	return -1;
}

int SetSerialDataBits(int bits)
{
	if (GetStatus())
	{
		SerialPortStruct.ByteSize = bits;

		return SetCommState(serial_port, &SerialPortStruct);
	}

	return 0;
}

int main()
{
	int i;
	serial_port = OpenSerial(3);
	if (serial_port == NULL) {
		printf ("Failed to open port %d\n", 3);
	}

	SetSerialBoudRate(115200);
	SetSerialStopBits(1);
	SetSerialDataBits(8);
	SetSerialParity(0);

	char received_data[0x200];
	memset (received_data, 0, 0x200);
	

	if (ReceiveSerialData(received_data, 0x1b0) == FALSE)
	{
		printf ("Failed\n");
	}

	for (i = 0; i < 0x200; i++) {
		printf ("%2.2x", received_data[i]);
	}
	
	FILE *fa = fopen("dump.bin", "wb");
	fwrite (received_data, 0x200, 1, fa);
	fclose (fa);

	CloseHandle(serial_port);

	return 1;
}
